/*
 * faster_click.c
 *
 *  Created on: 16/mar/2014
 *      Author: fb
 *
 *  This program challenges the reaction times of two players by detecting which between KEY_PLAYER1 and KEY_PLAYER2
 *  switches is pressed first after turning on one of the GREEN LEDS. The faster click is indicated by turning on either
 *  the LEDR_PLAYER1_FAST or LEDR_PLAYER2_FAST RED LEDS.
 *  Which of the GREEN LEDS is turned on and the time interval between two consecutive trials is made "random".
 *  There is timeout after which all the GREN LEDS are turned on and the current trial is discarded.
 *  The scoring is shown on HEX_DISPLAY_RES 7-seg display.
 *  The game is started and stopped by the SLIDER_SWITCH_RUN slider.
 */


#include "../Inc/config.h"
#include <stdint.h>


/*
 * Function: HexToSevenSeg
 * -----------------------
 * Converts each of the 4 hex digit of a 16-bit number into the corresponding representation on the 7-seg display
 * shown below and assembles them into a 32-bit word that can be used to control a set of 4x 7-seg displays present
 * in DE boards
 *         0
 *       -----
 *     5 |___| 1
 *     4 | 6 | 2
 *       -----
 *         3
 */

uint32_t HexToSevenSeg(unsigned int hex) {
	const uint8_t LUT_HEX_TO_SEVEN_SEG[16] = {
	    0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7D, 0x07, 0x7F, 0x67, /* 0-9 */
	    0x77, 0x7C, 0x39, 0x5E, 0x79, 0x71 };                       /* A,b,C,d,E,F */

	uint32_t data = LUT_HEX_TO_SEVEN_SEG[hex            & 15]        |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >> 4) & 15] <<  8) |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >> 8) & 15] << 16) |
			           (LUT_HEX_TO_SEVEN_SEG[(hex >>12) & 15] << 24);

    return data;
}
/* ------------------------------------------------------------------------- */

/* delay generation */
#define CYCLES_PER_MS 254 /* hand tuned to achieve around 1 ms resolution (code optimization OFF)*/
void Wait_ms(unsigned int time_ms) {
	int i,j;
	for(j=0; j<time_ms; j++) {
		for(i=0; i<CYCLES_PER_MS; i++) {;}
	}
}
/* ------------------------------------------------------------------------- */


int main() {

	static uint32_t counter_rand = 0, counter_click;
	static uint8_t run = 0, rand;
	static uint8_t score1, score2, buttons;

	while(1) {
		// 4th LSB of counter_rand used as random number
		counter_rand++;
		// check if SLIDER_SWITCH RUN is on
		if(SLIDER_SWITCHES_DATA_REG & (1<<SLIDER_SWITCH_RUN)) {
			if(!run) {
				// turn on LEDR_RUN, resets scores
				run = 1;
				RED_LEDS_DATA_REG |= (1<<LEDR_RUN);			// turn on LEDR_RUN
				score1 = score2 = 0;
				HEX_DISPLAY_RES = HexToSevenSeg((score1<<8) | score2);	// clear score hex display
			}

			rand = counter_rand & MASK_COUNTER;

			Wait_ms(WAIT_MIN + WAIT_RAND*rand);

			counter_click = 0;
			//PUHBUTTONS_EDGE_CAPTURE_REG = (1<<KEY_PLAYER1) | (1<<KEY_PLAYER2);
			/* Clear edge capture register of PUSHBUTTONS Parallel Port
			 * Writing to this register clears all register bits */
			PUHBUTTONS_EDGE_CAPTURE_REG = 0;
			GREEN_LEDS_DATA_REG |= (1 << rand);
			// wait for a key pressure
			while(!((buttons=(PUHBUTTONS_EDGE_CAPTURE_REG & ((1<<KEY_PLAYER1) | (1<<KEY_PLAYER2)))) ||
				    (counter_click > MAX_COUNTER_CLICK))) {
				counter_click++;
				counter_rand++;
			}
			GREEN_LEDS_DATA_REG &= ~(1 << rand);
			if(!buttons) {
				// no click
				GREEN_LEDS_DATA_REG |= MSK_GREEN_LEDS_NO_CLICK;
				Wait_ms(WAIT_LEDR_ON);
				GREEN_LEDS_DATA_REG &= ~MSK_GREEN_LEDS_NO_CLICK;
			}
			else if ((buttons & (1<<KEY_PLAYER1)) && !(buttons & (1<<KEY_PLAYER2))) {
				// player1 faster
				score1++;
				HEX_DISPLAY_RES = HexToSevenSeg((score1<<8) | score2);
				RED_LEDS_DATA_REG |= (1<<LEDR_PLAYER1_FAST);
				Wait_ms(WAIT_LEDR_ON);
				RED_LEDS_DATA_REG &= ~(1<<LEDR_PLAYER1_FAST);
			}
			else if (!(buttons & (1<<KEY_PLAYER1)) && (buttons & (1<<KEY_PLAYER2))) {
				// player2 faster
				score2++;
				HEX_DISPLAY_RES = HexToSevenSeg((score1<<8) | score2);
				RED_LEDS_DATA_REG |= (1<<LEDR_PLAYER2_FAST);
				Wait_ms(WAIT_LEDR_ON);
				RED_LEDS_DATA_REG &= ~(1<<LEDR_PLAYER2_FAST);
			}
			else {
				RED_LEDS_DATA_REG |= (1<<LEDR_PLAYER1_FAST) | (1<<LEDR_PLAYER2_FAST);
				Wait_ms(WAIT_LEDR_ON);
				RED_LEDS_DATA_REG &= ~((1<<LEDR_PLAYER1_FAST) | (1<<LEDR_PLAYER2_FAST));
			}
		}
		else {
			run = 0;
			RED_LEDS_DATA_REG &= ~(1<<LEDR_RUN);			// turn off LEDR_RUN
		}
	}// while
}// main

